#include "ezwin.h"
#include "stoplite.h"
using namespace std ;

// This must be imported.
StopLight mySignal ;

SimpleWindow myWin("StopLight", 12.0f, 15.0f) ;

struct LightInfo {
	Position cornBoxUL ;
	Position cornBoxLR ;
	Position cornGreenUL ;
	Position cornGreenLR ;
	Position cornYellowUL ;
	Position cornYellowLR ;
	Position cornRedUL ;
	Position cornRedLR ;
	Position cornTextUL ;
	Position cornTextLR ;
	char *label ;
	int stateGreen ;
	int stateYellow ;
	int stateRed ;
} ;

struct LightInfo NS, EW ;

void DrawLight(struct LightInfo &l,
			   int stateRed,
			   int stateYellow,
			   int stateGreen) {
	if (l.stateRed != stateRed) {
		l.stateRed = stateRed ;
		if (stateRed)
			myWin.RenderEllipse(l.cornRedUL, l.cornRedLR, Red, true ) ;
		else
			myWin.RenderEllipse(l.cornRedUL, l.cornRedLR, Black, true ) ;
	}
	if (l.stateYellow != stateYellow) {
		l.stateYellow = stateYellow ;
		if (stateYellow)
			myWin.RenderEllipse(l.cornYellowUL, l.cornYellowLR, Yellow, true ) ;
		else
			myWin.RenderEllipse(l.cornYellowUL, l.cornYellowLR, Black, true ) ;
	}
	if (l.stateGreen != stateGreen) {
		l.stateGreen = stateGreen ;
		if (stateGreen)
			myWin.RenderEllipse(l.cornGreenUL, l.cornGreenLR, Green, true) ;
		else
			myWin.RenderEllipse(l.cornGreenUL, l.cornGreenLR, Black, true) ;
	}
}

void ComputeBulbPosition(struct LightInfo &l) {
	float radBulb = 0.4*(l.cornBoxLR.GetXDistance() - l.cornBoxUL.GetXDistance()) ;
	float midX = (l.cornBoxUL.GetXDistance() + l.cornBoxLR.GetXDistance()) * 0.5 ;

	float midY = (l.cornBoxUL.GetYDistance() + l.cornBoxLR.GetYDistance()) * 0.5 ;
	l.cornYellowUL.SetXDistance(midX - radBulb) ;
	l.cornYellowUL.SetYDistance(midY - radBulb) ;
	l.cornYellowLR.SetXDistance(midX + radBulb) ;
	l.cornYellowLR.SetYDistance(midY + radBulb) ;

	float topY = (midY - radBulb + l.cornBoxUL.GetYDistance()) * 0.5 ;
	l.cornRedUL.SetXDistance(midX - radBulb) ;
	l.cornRedUL.SetYDistance(topY - radBulb) ;
	l.cornRedLR.SetXDistance(midX + radBulb) ;
	l.cornRedLR.SetYDistance(topY + radBulb) ;

	float botY = (midY + radBulb + l.cornBoxLR.GetYDistance()) * 0.5 ;
	l.cornGreenUL.SetXDistance(midX - radBulb) ;
	l.cornGreenUL.SetYDistance(botY - radBulb) ;
	l.cornGreenLR.SetXDistance(midX + radBulb) ;
	l.cornGreenLR.SetYDistance(botY + radBulb) ;

}

int RefreshStopLight()
{
	float winX = myWin.GetWidth() ;
	float winY = myWin.GetHeight() ;

	float lightSizeX = winX*0.5 ;
	float lightSizeY = winY - 1.25 ;
	if (lightSizeY > 3.0*lightSizeX)
		lightSizeY = 3.0*lightSizeX ;
	else
		lightSizeX = lightSizeY*0.33 ;

	float light1ULX = (winX*0.5 - lightSizeX)*0.5 ;
	float light2ULX = light1ULX + winX/2.0 ;
	NS.cornBoxUL.SetXDistance((winX*0.5 - lightSizeX)*0.5) ;
	NS.cornBoxUL.SetYDistance((winY - 1.0 - lightSizeY)*0.5 + 0.25) ;
	EW.cornBoxUL.SetXDistance(NS.cornBoxUL.GetXDistance() + winX/2.0) ;
	EW.cornBoxUL.SetYDistance(NS.cornBoxUL.GetYDistance()) ;
	NS.cornBoxLR.SetXDistance(NS.cornBoxUL.GetXDistance() + lightSizeX) ;
	NS.cornBoxLR.SetYDistance(NS.cornBoxUL.GetYDistance() + lightSizeY) ;
	EW.cornBoxLR.SetXDistance(EW.cornBoxUL.GetXDistance() + lightSizeX) ;
	EW.cornBoxLR.SetYDistance(EW.cornBoxUL.GetYDistance() + lightSizeY) ;
	myWin.RenderRectangle(NS.cornBoxUL, NS.cornBoxLR, Blue, true ) ;
	myWin.RenderRectangle(EW.cornBoxUL, EW.cornBoxLR, Blue, true ) ;
	NS.cornTextUL.SetXDistance(0.0f) ;
	NS.cornTextUL.SetYDistance(NS.cornBoxLR.GetYDistance()) ;
	NS.cornTextLR.SetXDistance(winX*0.5) ;
	NS.cornTextLR.SetYDistance(winY) ;
	EW.cornTextUL.SetXDistance(winX*0.5) ;
	EW.cornTextUL.SetYDistance(EW.cornBoxLR.GetYDistance()) ;
	EW.cornTextLR.SetXDistance(winX) ;
	EW.cornTextLR.SetYDistance(winY) ;
	if (winX > 5.0) {
		NS.label = "North South" ;
		EW.label = "East West" ;
	}
	else {
		NS.label = "N S" ;
		EW.label = "E W" ;
	}
	myWin.RenderText(NS.cornTextUL, NS.cornTextLR, NS.label) ;
	myWin.RenderText(EW.cornTextUL, EW.cornTextLR, EW.label) ;
	ComputeBulbPosition(NS) ;
	ComputeBulbPosition(EW) ;
	NS.stateRed    = -1 ;
	NS.stateYellow = -1 ;
	NS.stateGreen  = -1 ;
	EW.stateRed    = -1 ;
	EW.stateYellow = -1 ;
	EW.stateGreen  = -1 ;
	DrawLight(NS,
		      mySignal.bulbNSR(), mySignal.bulbNSY(), mySignal.bulbNSG()) ;
	DrawLight(EW,
		      mySignal.bulbEWR(), mySignal.bulbEWY(), mySignal.bulbEWG()) ;
	return 0 ;

}

int TickStopLight() {
	mySignal.Tick() ;
	myWin.StartTimer(100) ;
	DrawLight(NS,
		      mySignal.bulbNSR(), mySignal.bulbNSY(), mySignal.bulbNSG()) ;
	DrawLight(EW,
		      mySignal.bulbEWR(), mySignal.bulbEWY(), mySignal.bulbEWG()) ;
	return 0 ;
}

int ApiMain() 
{
	myWin.Open() ;
	myWin.SetRefreshCallback(RefreshStopLight) ;
	RefreshStopLight() ;
	myWin.SetTimerCallback(TickStopLight) ;
	myWin.StartTimer(100) ;
	return 0 ;
}

int ApiEnd()
{
	myWin.Close() ;
	return 0 ;
}
